#include "./include/runtime.h"
#include "./include/obj.h"

extern bool JS_ERR_FLAG = false;

js_value_t js_mk_num(double val) {
	return (js_value_t){ .type = JS_T_NUM, .data = *(void**)&val };
}
js_value_t js_mk_bool(bool val) {
	return val ? JS_TRUE : JS_FALSE;
}

js_value_t js_mk_str(js_string_t str) {
	js_gc_str_t res = malloc(sizeof *res);
	res->str = str;
	res->gc = 0;

	return (js_value_t){ .data = res, .type = JS_T_STR };
}
void js_str_free(js_gc_str_t str) {
	if (!str->str.dynamic || str->gc.refs == 0 || str == NULL) return;

	str->gc.refs--;
	if (str->gc.refs == 0) {
		free(str->str.data);
		free(str);
	}
}

js_value_t js_mk_symbol(js_string_t str) {
	js_gc_str_t res = malloc(sizeof *res);
	res->str = str;
	res->gc.refs = 0;

	return (js_value_t){ .data = res, .type = JS_T_STR };
}

js_value_t js_mk_obj(js_ctx_t ctx) {
	js_obj_t obj = malloc(sizeof *obj);

	obj->buckets_n = 16;
	obj->buckets = malloc(sizeof *obj->buckets * 16);
	obj->first = NULL;
	obj->gc.refs = 0;

	return (js_value_t){ .type = JS_T_OBJ, .data = obj };
}
void js_obj_free(js_gc_str_t str) {
	if (!str->str.dynamic || str->gc.refs == 0 || str == NULL) return;

	str->gc.refs--;
	if (str->gc.refs == 0) {
		free(str->str.data);
		free(str);
	}
}

void js_bootstrap(int argc, const char **argv) {
	
}

typedef struct {
	js_gc_table_node_t next;
} js_gc_table_node_t;